use axum::{Router, extract::Form, http::StatusCode, routing::post};
use color_eyre::eyre::{self, Context};
use dotenvy_macro::dotenv;
use error::WithStatusCode;
use serde::Deserialize;
use serenity::all::{ExecuteWebhook, Http, Webhook};
use tracing::instrument;
use tracing_error::ErrorLayer;
use tracing_subscriber::{EnvFilter, layer::SubscriberExt, util::SubscriberInitExt};

mod error;

const WEBHOOK: &'static str = dotenv!("WEBHOOK");
const LISTEN_ADDR: &'static str = dotenv!("LISTEN_ADDR");

#[tokio::main]
async fn main() -> eyre::Result<()> {
    color_eyre::install()?;

    tracing_subscriber::registry()
        .with(ErrorLayer::default())
        .with(
            EnvFilter::try_from_default_env()
                .or_else(|_| EnvFilter::try_new("info"))
                .unwrap(),
        )
        .with(tracing_subscriber::fmt::layer())
        .init();

    let app = Router::new().route("/api/speculate", post(speculate));

    let listener = tokio::net::TcpListener::bind(LISTEN_ADDR)
        .await
        .wrap_err("Failed to listen on that port")?;
    axum::serve(listener, app)
        .await
        .wrap_err("Failed to run axum service")?;
    Ok(())
}

#[derive(Deserialize, Debug)]
struct Speculation {
    speculation: String,
}

#[instrument]
async fn speculate(Form(speculation): Form<Speculation>) -> Result<maud::Markup, error::Error> {
    let discord_http = Http::new("");

    let webhook = Webhook::from_url(&discord_http, WEBHOOK)
        .await
        .wrap_err("Failed to initialize webhook")
        .with_status_code(StatusCode::INTERNAL_SERVER_ERROR)?;

    let builder = ExecuteWebhook::new()
        .content(speculation.speculation)
        .username("Anonymous speculator");
    webhook
        .execute(&discord_http, false, builder)
        .await
        .wrap_err("Could not execute webhook")
        .with_status_code(StatusCode::INTERNAL_SERVER_ERROR)?;

    Ok(maud::html! {
        p { "Speculation launched at high speed directly into our DMs" }
    })
}
